package com.nfllab.android.avexample;

import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;

public class PackageReceiver extends BroadcastReceiver {

	private class ScanResult {
		String VirusName;
		String ErrorStr;
	}

	private String GetAPKPath(Context context, Uri packageUri) {
		String APKPath = null;
		PackageManager pm = context.getPackageManager();
		try {
			PackageInfo pi = pm.getPackageInfo(packageUri.getSchemeSpecificPart(), 0);
			APKPath = pi.applicationInfo.sourceDir;
		} catch (NameNotFoundException e) {
			// return null
		}
		return APKPath;
	}
	
	private ScanResult ScanAPK(String fileName) {
		ScanResult result = new ScanResult();
		String eicar = "X5O!P%@AP[4\\PZX54(P^)7CC)7}$EICAR-STANDARD-ANTIVIRUS-TEST-FILE!$H+H*";
		byte[] buffer = new byte[130];
		try {
			ZipFile zip = new ZipFile(fileName);
			Enumeration<? extends ZipEntry> entries = zip.entries();
			while (entries.hasMoreElements()) {
				ZipEntry entry = (ZipEntry)entries.nextElement();
				try {
					InputStream is = zip.getInputStream(entry);
					int len = is.read(buffer); // FIXME: read repeatedly
					if (len >= 68 && len <= 128)
					{
						String s = new String(buffer, 0, 68);
						if (s.equals(eicar)) 
						{
							result.VirusName = "EICAR TEST FILE";
						}
					}
				} catch (IOException e) {
					result.ErrorStr = "INTERNAL ERROR OPENING " + entry.getName() + " IN " + fileName;
				}
			}
		} catch (IOException e) {
			result.ErrorStr = "INTERNAL ERROR OPENING " + fileName;
		}
		return result;
	}
	
	private void SendNotification(Context context, Uri packageUri, ScanResult result) {
		String output;
		Intent notificationIntent = null;
		if (result.ErrorStr != null) {
			output = "Error: " + result.ErrorStr;
		} else if (result.VirusName != null) {
			output = "Infected with " + result.VirusName;
			notificationIntent = new Intent(Intent.ACTION_DELETE, packageUri);
		} else {
			output = "Clean file";
		}
		NotificationManager mNotificationManager =
			(NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);
		Notification notification = new Notification(android.R.drawable.stat_notify_error,
				"AVExample finished scanning", System.currentTimeMillis());
		PendingIntent contentIntent = PendingIntent.getActivity(context, 0, notificationIntent, 0);
		notification.setLatestEventInfo(context, output, packageUri.getSchemeSpecificPart(), contentIntent);
		notification.flags |= Notification.FLAG_AUTO_CANCEL;
		mNotificationManager.notify(0, notification);
	}
	
	@Override
	public void onReceive(Context context, Intent intent) {
		if(Intent.ACTION_PACKAGE_ADDED.equals(intent.getAction())) {
			String APKPath = GetAPKPath(context, intent.getData());
			if (APKPath != null) {
				// TODO: move scanning from main thread to service
				SendNotification(context, intent.getData(), ScanAPK(APKPath));
			}
		}
	}
}
